# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require 'sqlite3'
require 'singleton'

module NihongoBenkyo

class Database 
    include Singleton
        
    def initialize 
        open
        tables_create unless tables_exist?
        
        @db.busy_handler do |data, retries|
            sleep(1)
        end
    end
    
    def open
        @db = SQLite3::Database.new(NihongoBenkyo::DB_PATH)    
    end
    
    def close
        @db.close
    end
       
    def search(word, &block)
        if word.latin?
            # from romaji
            search_kanji(word.to_kana, block)
            # from translation
            search_kana_kanji(word, block)
        elsif word.kana?
            search_kanji(word, block)
        elsif word.japanese?
            search_kana(word, block)
        end
    end
    
    def search_translations(id)
        @db.execute("SELECT translation FROM translations WHERE entry_id = #{id}") do |row|
            yield(row[0])
        end
    end
       
    def dictionary_list
        array = []
        @db.execute("SELECT selected, name FROM dictionaries ORDER BY position ASC") do |row|
            array << {"selected" => row[0].to_i, "dicname" => row[1] }
        end
        array
    end
    
    def dictionary_exists?(dicname)
        n = @db.get_first_value("SELECT count(*) FROM dictionaries WHERE name = '#{dicname}'")
        n == "0" ? false : true
    end
    
    def dictionary_remove(dicname)
        did = @db.get_first_value("SELECT id FROM dictionaries WHERE name = '#{dicname}'").to_i
        @db.transaction
            entries = "(SELECT id FROM entries WHERE dictionary_id = #{did})"
            @db.execute("DELETE FROM dictionaries WHERE id = #{did}")
            @db.execute("DELETE FROM kanji WHERE entry_id IN #{entries}")
            @db.execute("DELETE FROM kana_readings WHERE entry_id IN #{entries}")
            @db.execute("DELETE FROM translations WHERE entry_id IN #{entries}")   
            @db.execute("DELETE FROM entries WHERE dictionary_id = #{did}")
        @db.commit
    end
    
    def dictionary_select(dicname, selected=1)
        @db.execute("UPDATE dictionaries SET selected = #{selected} WHERE name = '#{dicname}'")
    end
    
    def dictionary_unselect(dicname)
        dictionary_select(dicname, 0)
    end
    
    def dictionary_change_name(oldname, newname)
        @db.execute("UPDATE dictionaries SET name = '#{newname}' WHERE name = '#{oldname}'")
    end
    
    def dictionary_change_position(old_path, new_path)
        @db.transaction
            @db.execute("UPDATE dictionaries set position = -1 WHERE position = #{new_path}")
            @db.execute("UPDATE dictionaries set position = #{new_path} WHERE position = #{old_path}")
            @db.execute("UPDATE dictionaries set position = #{old_path} WHERE position = -1")
        @db.commit
    end
    
    def get_config(name)
        row = @db.get_first_row("SELECT type, value FROM config WHERE name = '#{name}'")
        
        if row.nil?
            if Config::DEFAULT.has_key?(name)
                Config::DEFAULT[name]        
            else
                raise "No config element for #{name}"
            end
        else
            if row[0] == "TrueClass"
                true
            elsif row[0] == "FalseClass"
                false
            elsif row[0] == "String"
                row[1]
            end
        end
    end
    
    def set_config(name, value)
        class_name = value.class.to_s
        
        if class_name == "TrueClass" or class_name == "FalseClass"
            sql = "INSERT OR REPLACE INTO config (name, type) "
            sql += "VALUES ('#{name}', '#{class_name}')"
            @db.execute(sql)
        elsif class_name == "String"
            sql = "INSERT OR REPLACE INTO config (name, type, value) "
            sql += "VALUES ('#{name}', '#{class_name}', '#{value}')"
            @db.execute(sql) 
        end
    end
    
    private
    
    def tables_exist?
        n = @db.get_first_value("SELECT count(*) FROM sqlite_master WHERE type = 'table' and name = 'entries'")
        n == "0" ? false : true
    end
    
    def tables_create
        path = File.join(NihongoBenkyo::Config::DATA_DIR, "sql", "db.sql")
        file = File.open(path, "r")
        @db.execute_batch(file.readlines.join("\n"))
        file.close
    end
    
    def search_kanji(kana, block)
        
        subquery = "SELECT k.entry_id FROM kana_readings k, entries e, dictionaries d "
        subquery += "WHERE k.kana_reading = '#{kana}' "
        subquery += "AND k.entry_id = e.id AND e.dictionary_id = d.id AND d.selected = 1"
        
        query = "SELECT k.entry_id, k.kanji FROM kanji k, entries e, dictionaries d "
        query += "WHERE k.entry_id IN(#{subquery}) "
        query += "AND k.entry_id = e.id AND e.dictionary_id = d.id AND d.selected = 1 "
        query += "ORDER BY d.position ASC"
        
        @db.execute(query) do |row|
            block.call(row[0], kana, row[1])
        end
    end
        
    def search_kana(kanji, block)

        subquery = "SELECT k.entry_id FROM kanji k, entries e, dictionaries d "
        subquery += "WHERE k.kanji = '#{kanji}' "
        subquery += "AND k.entry_id = e.id AND e.dictionary_id = d.id AND d.selected = 1"
        
        query = "SELECT k.entry_id, k.kana_reading FROM kana_readings k, entries e, dictionaries d "
        query += "WHERE k.entry_id IN(#{subquery}) "
        query += "AND k.entry_id = e.id AND e.dictionary_id = d.id AND d.selected = 1 "
        query += "ORDER BY d.position ASC"
        
        @db.execute(query) do |row|
            block.call(row[0], row[1], kanji)
        end
    end
    
    def search_kana_kanji(translation, block)

        subquery = "SELECT t.entry_id FROM translations t, entries e, dictionaries d "
        subquery += "WHERE t.translation = '#{translation}' "
        subquery += "AND t.entry_id = e.id AND e.dictionary_id = d.id AND d.selected = 1"
        
        query = "SELECT k.entry_id, k.kana_reading, kanji.kanji "
        query += "FROM kana_readings k, kanji, entries e, dictionaries d "
        query += "WHERE k.entry_id IN(#{subquery}) "
        query += "AND k.entry_id = e.id  AND kanji.entry_id = k.entry_id "
        query += "AND e.dictionary_id = d.id AND d.selected = 1 "
        query += "ORDER BY d.position ASC"
        
        @db.execute(query) do |row|
            block.call(row[0], row[1], row[2])
        end
    end

end

end