# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo
module UI

    class MainApp < GladeBase 
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")
        
        module Columns
            ID, KANJI, KANA, ROMAJI, TRANSLATIONS = (0..5).to_a
        end

        def initialize
            super("main_app.glade")
            @prefs = Preferences.instance
            @db = Database.instance
            
            initialize_ui
            initialize_signals
            load_preferences
            scan_selection
        end
                       
        def on_search
        
            # First ensure the word is OK
            word = @search_combobox_entry.child.text
            word = SQLite3::Database.quote(word)
            word.strip!
            
            # Save the words
            @search_combobox_entry.append_text(word)
            
            if @search_combobox_entry.model.nb_rows > 10
                @search_combobox_entry.model.remove_last
            end
            
            # The search properly speaking
            @result_liststore.clear
            
            n = 0               
            @db.search(word) do |entry_id, kana, kanji|
                row = @result_liststore.append()
                row[Columns::KANJI] = kanji
                row[Columns::KANA] = kana
                row[Columns::ROMAJI] = kana.to_roma
                
                translations = ""
                @db.search_translations(entry_id) do |translation|
                    translations += translation + " / "
                end
                
                row[Columns::TRANSLATIONS] = translations
                
                n += 1
            end
            
            @result_listview.columns_autosize
            
            # Status bar
            @statusbar.push(0, _("Matches found: ") + n.to_s)
        end
        
        def on_quit
            save_preferences
            Gtk.main_quit
        end
        
        def on_settings
            PreferencesDialog.new(@main_app) do
                set_visible_columns
            end
        end
        
        def on_about
            AboutDialog.new(@main_app).show
        end
        
                
        private
        
        def scan_selection
            @thread = Thread.new(@search_combobox_entry) do |combobox|
                
                last_selection = nil
                Gtk::Clipboard.get(Gdk::Selection::PRIMARY).request_text do |clipboard, text|
                    last_selection = text
                end
                
                while true
                    Gtk::Clipboard.get(Gdk::Selection::PRIMARY).request_text do |clipboard, text| 
                        if text != last_selection
                            unless text.nil? or not(@scan_selection.active?)                    
                                combobox.child.text = last_selection = text 
                                on_search
                                @main_app.present 
                            end
                        end
                    end
                    sleep(0.8)
                end
            end
        end
        
        def initialize_signals       
            @search_combobox_entry.child.signal_connect("activate") do 
                on_search
            end
            
            @search_menu_item.signal_connect("select") do 
                @search_combobox_entry.child.grab_focus
            end
            
            @view_statusbar.signal_connect("toggled") do 
                 @statusbar.visible = @view_statusbar.active?                         
            end
            
        end
        
        def set_visible_columns
            cols_prefs = [
                @prefs.col_kanji_visible,
                @prefs.col_kana_visible,
                @prefs.col_romaji_visible,
                @prefs.col_translations_visible
            ]
            
            cols = @result_listview.columns
            
            cols.each_index do |i|
                cols[i].visible = cols_prefs[i]
            end
            
            @result_listview.headers_visible = @prefs.headers_visible
        end
        
        def initialize_result_listview
            # id, kanji, kana, romaji, translations
            @result_liststore = Gtk::ListStore.new(Fixnum,String,String,String,String)
            @result_listview.model = @result_liststore
            @result_listview.selection.mode = Gtk::SELECTION_SINGLE
            @result_listview.headers_visible = true
            
            columns = [
                ["Kanji", Columns::KANJI],
                ["Kana", Columns::KANA],
                ["Romaji", Columns::ROMAJI],
                [_("Translations"), Columns::TRANSLATIONS]
            ]
            
            columns.each do |column_name, column_id|
                renderer = Gtk::CellRendererText.new
                col = Gtk::TreeViewColumn.new(column_name, renderer, :text => column_id) 
                col.sort_column_id = column_id
                col.resizable = true
                @result_listview.append_column(col)
            end
            
            set_visible_columns
        end
        
        def initialize_ui
            combobox_list = Gtk::ListStore.new(String)
            @search_combobox_entry.model = combobox_list
            @search_combobox_entry.text_column = 0
            initialize_result_listview
        end
        
        def load_preferences
            @view_statusbar.active = @prefs.view_statusbar
            @scan_selection.active = @prefs.scan_selection
            
            @prefs.last_words.split("\n").reverse_each do |word|
                @search_combobox_entry.append_text(word)
            end
        end
        
        def save_preferences
            @prefs.view_statusbar = @view_statusbar.active?
            @prefs.scan_selection = @scan_selection.active?
            
            last_words = []
            @search_combobox_entry.model.each do |model, path, iter|
                last_words << iter[0]    
            end
            @prefs.last_words = last_words.join("\n")
        end
        
    end
end
end