# Nihongo Benkyo
# Copyright (C) 2005 Mathieu Blondel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

module NihongoBenkyo
module UI

    class PreferencesDialog < GladeBase 
        include GetText
        GetText.bindtextdomain(NihongoBenkyo::TEXTDOMAIN, nil, nil, "UTF-8")

        def initialize(parent, &changed_block)
            super("preferences_dialog.glade")
            @preferences_dialog.transient_for = parent
            @preferences = Preferences.instance
            @db = Database.instance
            @changed_block = changed_block
            @add_dictionary_button.sensitive = true if import_exists?
            init_dictionary_list_view
            
            @db.dictionary_list.each do |h| 
                append_dictionary(h["dicname"],h["selected"])
            end
                       
            @cols = {
                @checkbutton_headers => "headers_visible",
                @checkbutton_col_kanji => "col_kanji_visible",
                @checkbutton_col_kana => "col_kana_visible",
                @checkbutton_col_romaji => "col_romaji_visible",
                @checkbutton_col_translations => "col_translations_visible"
            }
            
            @cols.each_pair do |checkbutton, pref_name|
                checkbutton.active = @preferences.send(pref_name)
            end
        end
        
        def on_preferences_close
            @preferences_dialog.destroy
        end
               
        def on_add_dictionary
            AddDictionaryDialog.new(self)
        end
        
        def on_remove_dictionary
            thread = Thread.new do
                selection = @dictionary_treeview.selection.selected
                dicname = selection[1]
                unless selection.nil?
                    @list_store.remove(selection) 
                    @db.dictionary_remove(dicname)
                end
            end
            
            pbar = ProgressBar.new(_("Removing dictionary..."), @preferences_dialog)       
            while thread.alive?
                pbar.pulse   
            end
            pbar.destroy
        end
        
        def on_dictionary_up
            iter = @dictionary_treeview.selection.selected
            old_path = iter.path
            new_path = iter.path
            new_path.prev!
            model = @dictionary_treeview.model
            model.move_after(model.get_iter(new_path), iter)
            @db.dictionary_change_position(old_path, new_path)
            sensitize_buttons
        end

        def on_dictionary_down
            iter = @dictionary_treeview.selection.selected
            next_iter = iter.dup
            old_path = next_iter.path
            next_iter.next!
            new_path = next_iter.path
            @dictionary_treeview.model.move_after(iter, next_iter)
            @db.dictionary_change_position(old_path, new_path)
            sensitize_buttons
        end
        
        def on_column_toggled(checkbutton)
            @preferences.send("#{@cols[checkbutton]}=", 
                                      checkbutton.active?)
            @changed_block.call
        end
        
        def append_dictionary(dicname,selected=1)
            row = @list_store.append()
            row[0] = selected
            row[1] = dicname
        end   
        
        def preferences_dialog
            @preferences_dialog  
        end
        
        private
        
        def import_exists?
            ENV['PATH'].split(":").find do |dir|
                FileTest.executable?(File.join(dir,"nihongobenkyo-import"))
            end
        end
        
        def init_dictionary_list_view
            @list_store = Gtk::ListStore.new(Fixnum,String)
                        
            @dictionary_treeview.model = @list_store
            @dictionary_treeview.selection.mode = Gtk::SELECTION_SINGLE
            
            renderer = Gtk::CellRendererToggle.new
            col = Gtk::TreeViewColumn.new("Active", renderer, :active => 0)
            @dictionary_treeview.append_column(col)
            
            renderer.signal_connect("toggled") do |toggled,row_iter| 
                iter = @list_store.get_iter(row_iter)
                selected = iter[0]
                dicname = iter[1]
                if(selected == 1)
                    iter[0] = 0
                    @db.dictionary_unselect(dicname)
                else
                    iter[0] = 1
                    @db.dictionary_select(dicname)
                end                
            end

            renderer = Gtk::CellRendererText.new
            renderer.editable = true
            col = Gtk::TreeViewColumn.new("Dictionary", renderer, :text => 1)
            @dictionary_treeview.append_column(col) 
            
            renderer.signal_connect("edited") do |entry,row_iter,value|
                @db.dictionary_change_name(@list_store.get_iter(row_iter)[1], value)
                @list_store.get_iter(row_iter)[1] = value
            end
                      
            sensitize_buttons
            
            @dictionary_treeview.selection.signal_connect("changed") do
                sensitize_buttons
            end
        end
        
        def sel_iter
            @dictionary_treeview.selection.selected
        end
        
        def sensitize_buttons
            @remove_dictionary_button.sensitive = @dictionary_treeview.has_row_selected? 
            iter_last = @list_store.get_iter((@list_store.nb_rows - 1).to_s)
            
            @dictionary_up_button.sensitive = ((sel_iter != @list_store.iter_first) and (not sel_iter.nil?))
            @dictionary_down_button.sensitive = ((sel_iter != iter_last) and (not sel_iter.nil?))
        end
              
    end
        
end
end